import { SignalSourceEntity } from "./SignalSourceEntity";
import ApplicationConfigEntity from "./ApplicationConfigEntity";
import { WindowOpenNotifyEntity } from "./MultimediaWindowEntity";

export namespace Protocol {
  export class Commands {
    public static get PROTOCOL_PREFIX() {
      return "_";
    }

    public static get kUnKnowCommand() {
      return Commands.PROTOCOL_PREFIX + "UnKnowCommand";
    }

    public static get kSearchDevice() {
      return Commands.PROTOCOL_PREFIX + "SearchDevice";
    }
    public static get kEditNetworkInterfaceInfo() {
      return Commands.PROTOCOL_PREFIX + "EditNetworkInterfaceInfo";
    }
    public static get kExecCommand() {
      return Commands.PROTOCOL_PREFIX + "ExecCommand";
    }
    public static get kRestoreDevice() {
      return Commands.PROTOCOL_PREFIX + "RestoreDevice";
    }
    public static get kLogin() {
      return Commands.PROTOCOL_PREFIX + "Login";
    }
    public static get kLogout() {
      return Commands.PROTOCOL_PREFIX + "Logout";
    }
    public static get kRpcGetWindows() {
      return Commands.PROTOCOL_PREFIX + "RpcGetWindows";
    }
    public static get kRpcGetSignalSources() {
      return Commands.PROTOCOL_PREFIX + "RpcGetSignalSources";
    }

    public static get kRpcGetApplicationConfig() {
      return Commands.PROTOCOL_PREFIX + "RpcGetApplicationConfig";
    }

    public static get kMoveWindow() {
      return Commands.PROTOCOL_PREFIX + "MoveWindow";
    }

    public static get kResizeWindow() {
      return Commands.PROTOCOL_PREFIX + "ResizeWindow";
    }

    public static get kOpenWindow() {
      return Commands.PROTOCOL_PREFIX + "OpenWindow";
    }

    public static get kCloseWindow() {
      return Commands.PROTOCOL_PREFIX + "CloseWindow";
    }

    static _all_commands = new Set([
      Commands.kUnKnowCommand,
      Commands.kSearchDevice,
      Commands.kEditNetworkInterfaceInfo,
      Commands.kExecCommand,
      Commands.kRestoreDevice,
      Commands.kLogin,
      Commands.kLogout,
      Commands.kRpcGetWindows,
      Commands.kRpcGetSignalSources,
      Commands.kRpcGetApplicationConfig,
      Commands.kMoveWindow,
      Commands.kResizeWindow,
      Commands.kOpenWindow,
      Commands.kCloseWindow,
    ]);

    public static get AllCommands() {
      return this._all_commands;
    }
  }

  export class PacketEntity {
    public static get FLAG_REQUEST() {
      return 0;
    }
    public static get FLAG_RESPONSE() {
      return 1;
    }
    public static get FLAG_NOTIFY() {
      return 2;
    }

    /** 是否出现异常,如果是异常,则为 ExceptionEntity */
    has_exception = false;
    /** 0: 请求  1: 响应 */
    flag = PacketEntity.FLAG_REQUEST;
    /** rpc 消息 ID，默认 0 */
    rpc_id = 0;
    /** 命令 */
    command = Protocol.Commands.kUnKnowCommand;
  }

  export class LoginRequest extends PacketEntity {
    user_name = "";
    password = "";

    constructor(user_name?: string, password?: string) {
      super();
      this.user_name = user_name ?? "";
      this.password = password ?? "";
      this.command = Commands.kLogin;
    }
  }

  export class LoginResponse extends PacketEntity {
    success = false;
    error_code = 0;

    constructor() {
      super();
      this.command = Commands.kLogin;
    }
  }

  export class GetSignalSourcesRequest extends PacketEntity {
    timestamp = new Date().getMilliseconds();

    constructor(rpcid?: number) {
      super();
      this.rpc_id = rpcid ?? 0;
      this.command = Commands.kRpcGetSignalSources;
    }
  }

  export class GetSignalSourcesResponse extends PacketEntity {
    signal_sources: SignalSourceEntity[] = [];
    signal_source_groups: [] = [];

    constructor() {
      super();
      this.command = Commands.kRpcGetSignalSources;
    }
  }

  export class GetApplicationConfigRequestEntity extends PacketEntity {
    timestamp: number = new Date().getMilliseconds();

    constructor(rpcid?: number) {
      super();
      this.rpc_id = rpcid ?? 0;
      this.command = Commands.kRpcGetApplicationConfig;
    }
  }

  export class GetApplicationConfigResponseEntity extends PacketEntity {
    config: ApplicationConfigEntity | null = null;

    constructor() {
      super();
      this.command = Commands.kRpcGetApplicationConfig;
    }
  }

  export class GetWindowsRequestEntity extends PacketEntity {
    timestamp: number = new Date().getMilliseconds();

    constructor(rpcid?: number) {
      super();
      this.rpc_id = rpcid ?? 0;
      this.command = Commands.kRpcGetWindows;
    }
  }

  export class GetWindowsResponseEntity extends PacketEntity {
    windows: WindowOpenNotifyEntity[] = [];

    constructor() {
      super();
      this.command = Commands.kRpcGetWindows;
    }
  }

  export class MoveWindowRequestEntity extends PacketEntity {
    window_id: number = 0;
    x: number = 0;
    y: number = 0;

    constructor(window_id: number, x: number, y: number) {
      super();
      this.command = Commands.kMoveWindow;

      this.window_id = window_id ?? 0;
      this.x = x ?? 0;
      this.y = y ?? 0;
    }
  }

  export class ResizeWindowRequestEntity extends PacketEntity {
    window_id: number = 0;
    width: number = 0;
    height: number = 0;
    constructor(window_id: number, width: number, height: number) {
      super();
      this.command = Commands.kResizeWindow;
      this.window_id = window_id ?? 0;
      this.width = width ?? 0;
      this.height = height ?? 0;
    }
  }

  export class CloseWindowRequestEntity extends PacketEntity {
    window_id: number = 0;
    constructor(window_id: number) {
      super();
      this.command = Commands.kCloseWindow;
      this.window_id = window_id ?? 0;
    }
  }

  export class OpenWindowRequestEntity extends PacketEntity {
    signal_source: string = "";
    x: number = 0;
    y: number = 0;
    width: number = 0;
    height: number = 0;
    constructor() {
      super();
      this.command = Commands.kOpenWindow;
    }
  }

  export class WindowCloseNotifyEntity extends PacketEntity {
    window_id: number = 0;
    constructor() {
      super();
      this.command = Commands.kCloseWindow;
    }
  }

  export class WindowOpenNotifyEntity extends PacketEntity {
    window_id: number = 0;
    signal_source: string = "";
    x: number = 0;
    y: number = 0;
    width: number = 0;
    height: number = 0;
    constructor() {
      super();
      this.command = Commands.kOpenWindow;
    }
  }

  export class WindowMoveNotifyEntity extends PacketEntity {
    window_id: number = 0;
    x: number = 0;
    y: number = 0;
    constructor() {
      super();
      this.command = Commands.kMoveWindow;
    }
  }

  export class WindowResizeNotifyEntity extends PacketEntity {
    window_id: number = 0;
    width: number = 0;
    height: number = 0;
    constructor() {
      super();
      this.command = Commands.kResizeWindow;
    }
  }
}
